document.addEventListener('DOMContentLoaded', function() {

    // Delete language row
    document.addEventListener('click', function(event) {
        if (event.target.classList.contains('tlt-delete-language')) {
            event.preventDefault();
            var row = event.target.parentNode.parentNode;
            row.parentNode.removeChild(row);
        }
    });

    // Edit a row
    document.addEventListener('click', function(event) {
        if (event.target.classList.contains('tlt-edit-language')) {
            event.preventDefault();
            var row = event.target.parentNode.parentNode;
            var keyInput = row.querySelector('input[name="tlt-custom_languages_keys[]"]');
            var valueInput = row.querySelector('input[name="tlt-custom_languages_values[]"]');
            
            if (!row.classList.contains('tlt-editing')) {
                // Enter edit mode
                keyInput.removeAttribute('readonly');
                valueInput.removeAttribute('readonly');
                event.target.textContent = tltCustomLanguagesData.saveText;
                event.target.classList.add('tlt-save-language');
                event.target.classList.remove('tlt-edit-language');
                row.classList.add('tlt-editing');
            }
        } else if (event.target.classList.contains('tlt-save-language')) {
            event.preventDefault();
            var row = event.target.parentNode.parentNode;
            var keyInput = row.querySelector('input[name="tlt-custom_languages_keys[]"]');
            var valueInput = row.querySelector('input[name="tlt-custom_languages_values[]"]');
            if (row.classList.contains('tlt-editing')) {
                // Save changes
                var key = keyInput.value.trim();
                var value = valueInput.value.trim();
                // Remove spaces around commas in the value string
                value = value.replace(/\s*,\s*/g, ',');
                // Empty field
                if (key === '' || value === '') {
                    alert(tltCustomLanguagesData.alertText2);
                    return;
                }
                // Value without any coma
                if (value.indexOf(',') === -1) {
                    alert(tltCustomLanguagesData.alertText4);
                    return;
                }
                // Language code pattern check
                var pattern = /^[a-z]{2,3}(-[a-zA-Z0-9]+)*$/;
                if (!pattern.test(key)) {
                    alert(tltCustomLanguagesData.alertText5);
                    return;
                }
                // Assign the trimmed values back to the variables
                keyInput.value = key;
                valueInput.value = value;
                // Exit edit mode
                keyInput.setAttribute('readonly', true);
                valueInput.setAttribute('readonly', true);
                event.target.textContent = tltCustomLanguagesData.editText;
                event.target.classList.add('tlt-edit-language');
                event.target.classList.remove('tlt-save-language');
                row.classList.remove('tlt-editing');
            }
        }
    });

    // Cancel editing a row
    document.addEventListener('keydown', function(event) {
        if (event.key === 'Escape') {
            var editingRow = document.querySelector('.tlt-custom-language-row.tlt-editing');
            if (editingRow) {
                var keyInput = editingRow.querySelector('input[name="tlt-custom_languages_keys[]"]');
                var valueInput = editingRow.querySelector('input[name="tlt-custom_languages_values[]"]');
                keyInput.value = keyInput.getAttribute('value');
                valueInput.value = valueInput.getAttribute('value');
                keyInput.setAttribute('readonly', true);
                valueInput.setAttribute('readonly', true);
                var editButton = editingRow.querySelector('.tlt-save-language');
                editButton.textContent = tltCustomLanguagesData.editText;
                editButton.classList.remove('tlt-save-language');
                editButton.classList.add('tlt-edit-language');
                editingRow.classList.remove('tlt-editing');
            }
        }
    });

    // Add new language row
    var tltAddLanguageButton = document.getElementById('tlt-add-language');
    if (tltAddLanguageButton) {
        tltAddLanguageButton.addEventListener('click', function(event) {
            event.preventDefault();
            var container = document.getElementById('tlt-custom-languages-container');
            var newRow = document.createElement('div');
            newRow.classList.add('tlt-custom-language-row');
            newRow.innerHTML = '<div class="tlt-row-item"><label>'+tltCustomLanguagesData.keyText+'</label><input type="text" name="tlt-custom_languages_keys[]" value="" readonly=""></div><div class="tlt-row-item"><label>'+tltCustomLanguagesData.valueText+'</label><input type="text" class="tlt-field-value" name="tlt-custom_languages_values[]" value="" readonly=""></div><div class="tlt-row-item"><button type="button" class="tlt-edit-language button action">'+tltCustomLanguagesData.editText+'</button><button type="button" class="tlt-delete-language button action">'+tltCustomLanguagesData.deleteText+'</button></div>';
            container.appendChild(newRow);
            // Show the Save button
            var saveButton = document.getElementById('tlt-save-button');
            saveButton.style.display = 'inline-block';
        });
    }

    // Submit form
    var tltCustomLanguagesForm = document.getElementById('tlt-custom-languages-form');
    if (tltCustomLanguagesForm) {
        tltCustomLanguagesForm.addEventListener('submit', function(event) {
            event.preventDefault();
            var editableRows = document.querySelectorAll('.tlt-custom-language-row.tlt-editing');
            if (editableRows.length > 0) {
                alert(tltCustomLanguagesData.alertText1);
                return;
            }
            var emptyFields = document.querySelectorAll('.tlt-custom-language-row input[name="tlt-custom_languages_keys[]"], .tlt-custom-language-row input[name="tlt-custom_languages_values[]"]');
            for (var i = 0; i < emptyFields.length; i++) {
                if (emptyFields[i].value.trim() === '') {
                    event.preventDefault();
                    alert(tltCustomLanguagesData.alertText3);
                    return;
                }
            }
            // Perform additional validation or sanitization if required
            // ...
            this.submit();
        });
    }

    var tltSaveButton = document.getElementById('tlt-save-button');
    if (tltCustomLanguagesForm && tltSaveButton) {
        var languageRows = document.querySelectorAll('.tlt-custom-language-row');
        if (languageRows.length === 0) {
            // Hide the Save button
            tltSaveButton.style.display = 'none';
        }
    }
});