document.addEventListener('DOMContentLoaded', function() {
    var tltEditableCells = document.querySelectorAll('[contenteditable="true"]');
    var tltIsUpdating = false; // Flag to track ongoing updates
    var tltPopup = null; // Reference to the popup element
    var tltPopupTimeout = null; // Reference to the timeout for hiding the popup
    
    tltEditableCells.forEach(function(cell) {
        cell.addEventListener('blur', function() {
            clearTimeout(tltPopupTimeout); // Cancel the timeout for hiding the popup
            if (tltIsUpdating) {
                return; // Skip update if another update is already in progress
            }
            var originalValue = this.getAttribute('data-original');
            //var currentValue = this.textContent.trim();
            var currentValue = this.textContent;
            var language = this.getAttribute('data-language');
            var columnUpdated = this.getAttribute('column-updated');
            var beforeValue = this.getAttribute('before-value');
            var checkbox = this.closest('tr').querySelector('input[type="checkbox"]');
            var tltNonce = tltTranslationEditorData.tltTranslationEditorNonce;
                
            var currentValueDecoded = decodeEntities(currentValue);
            var beforeValueDecoded = decodeEntities(beforeValue);

            if (currentValueDecoded !== beforeValueDecoded) {
                // Perform AJAX update here
                var currentValue = currentValue.replace(/"/g, '&quote;'); // replace quote not to break the json data
                //var currentValue = currentValue.replace(/(?<=\s)&(?=\s)/g, ' &amp; '); // replace ' & '
                var data = {
                    currentValue: encodeURIComponent(currentValue),
                    originalValue: encodeURIComponent(originalValue),
                    language: language,
                    columnUpdated: columnUpdated,
                    tltNonce: tltNonce
                };
                //console.log('nonce: '+tltNonce);
                // Disable the cell during update
                this.setAttribute('contenteditable', 'false');
                // Show updating message
                showPopup('<div class="tlt-custom-spinner"></div>');
                var xhr = new XMLHttpRequest();
                xhr.open('POST', ajaxurl, true);
                //xhr.open('POST', tltTranslationEditorData.ajax_url, true);
                xhr.setRequestHeader('Content-Type', 'application/x-www-form-urlencoded; charset=UTF-8');
                xhr.onreadystatechange = function() {
                    if (xhr.readyState === XMLHttpRequest.DONE) {
                        tltIsUpdating = false; // Update ended
                        
                        if (xhr.status === 200) {
                            var response = JSON.parse(xhr.responseText);
                            var message = response.success ? tltTranslationEditorData.updateSuccessful : tltTranslationEditorData.updateFailed;

                            if (response.success) {
                                // Update the checkbox value attribute
                                checkbox.setAttribute('value', response.key);
                                // Select the source cell within the same row and set it to 0
                                var sourceCell = cell.closest('tr').querySelector('.source');
                                if (sourceCell) {
                                    sourceCell.textContent = '0';
                                }
                                // Update the translated_by element in the same row
                                var translatedByElement = cell.closest('tr').querySelector('.translated_by');
                                if (translatedByElement) {
                                    translatedByElement.textContent = response.user_id;
                                }
                                // Update the before-value attribute with the current value
                                cell.setAttribute('before-value', currentValue);
                                // Update the data-original attribute only for the original cell
                                if (columnUpdated === 'original') {
                                    cell.setAttribute('data-original', currentValue);
                                    // Update the data-original attribute for the translated cell
                                    // Select the next span element within the same row
                                    var translatedCell = cell.closest('tr').querySelector('.tlt-editable-cell[column-updated="translated"]');
                                    if (translatedCell) {
                                        translatedCell.setAttribute('data-original', currentValue);
                                    }
                                }
                            } else {
                                // Revert the cell content to the before-value attribute value
                                cell.textContent = cell.getAttribute('before-value');
                            }
                            showPopup(message);
                        } else {
                            // Update failed
                            cell.textContent = cell.getAttribute('before-value');
                            var message = tltTranslationEditorData.updateFailed;
                            showPopup(message);
                        }
                        // Re-enable the cell after update
                        cell.setAttribute('contenteditable', 'true');
                        // Hide the popup after a delay
                        tltPopupTimeout = setTimeout(function() {
                            hidePopup();
                        }, 2000);
                    }
                };
                tltIsUpdating = true; // Set the flag to indicate ongoing update
                xhr.send('action=update_translation&data=' + JSON.stringify(data));
            }
        });

        cell.addEventListener('keydown', function(event) {
            if (event.keyCode === 13 && !event.shiftKey) {
                // Enter key pressed without Shift key, prevent default behavior (newline) and blur the cell
                event.preventDefault();
                this.blur();
            } else if (event.keyCode === 27) {
                // Escape key pressed, cancel the edit
                this.textContent = this.getAttribute('before-value');
                this.blur();
            }
        });
        
        cell.addEventListener('paste', function(event) {
            // Prevent the default action
            event.preventDefault();

            // Get the copied text from the clipboard
            var text = event.clipboardData
                ? (event.originalEvent || event).clipboardData.getData('text/plain')
                : // For IE
                window.clipboardData
                ? window.clipboardData.getData('Text')
                : '';

            if (document.queryCommandSupported('insertText')) {
                document.execCommand('insertText', false, text);
            } else {
                // Insert text at the current position of caret
                var range = document.getSelection().getRangeAt(0);
                range.deleteContents();

                var textNode = document.createTextNode(text);
                range.insertNode(textNode);
                range.selectNodeContents(textNode);
                range.collapse(false);

                var selection = window.getSelection();
                selection.removeAllRanges();
                selection.addRange(range);
            }
        });

        function showPopup(content) {
            if (tltPopup) {
                // If popup already exists, update its content
                tltPopup.innerHTML = content;
            } else {
                // If popup doesn't exist, create a new one
                tltPopup = document.createElement('div');
                tltPopup.classList.add('tlt-popup');
                tltPopup.innerHTML = content;
                document.body.appendChild(tltPopup);
            }
            tltPopup.classList.remove('hide');
        }
        
        function hidePopup() {
            if (tltPopup) {
                tltPopup.classList.add('hide');
                setTimeout(function () {
                document.body.removeChild(tltPopup);
                tltPopup = null; // Set popup to null after removing it
                }, 500);
            }
        }
    });

    function decodeEntities(encodedString) {
        var textarea = document.createElement('textarea');
        textarea.innerHTML = encodedString;
        return textarea.value;
    }
});
