<?php
/**
 * About Class
 *
 * @package Transposh_Language_Toolbox
 */

namespace Transposh_Language_Toolbox;

class About {

    private $omahoung_default_url;

    /**
     * Initialize the About class.
     */
    public function init() {
        $this->get_omahoung_default_url();
        $this->render_about_page();
    }

    /**
     * Get the Omahoung default url based on user's preferred locale.
     */
    function get_omahoung_default_url() {
        $user_locale = get_user_locale();
        $default_url = 'https://www.omahoung.com/en/';
        // Check if the user's locale contains "zh" (Chinese)
        if ( strpos( $user_locale, 'zh' ) !== false ) {
            $default_url = 'https://www.omahoung.com/';
        }
        $this->omahoung_default_url = $default_url;
    }

    /**
     * Render the About & Support page content.
     */
    public function render_about_page() {
        ?>
        <div class="tlt-wrap">

            <div class="tlt-menu">
                <a href="#" class="tlt-menu-item active" data-tab="tlt-about-support"><?php esc_html_e( 'About & Support', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ); ?></a>
                <span class="tlt-menu-divider">|</span>
                <a href="#" class="tlt-menu-item" data-tab="tlt-privacy-policy"><?php esc_html_e( 'Privacy Policy', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ); ?></a>
            </div>

            <!-- About & Support -->
            <div id="tlt-about-support" class="tlt-tab-content active">
                <div class="tlt-section">
                    <h2><?php esc_html_e( 'About the Plugin', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ); ?></h2>
                    <p><?php esc_html_e( 'This plugin is developed by Omahoung.', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ); ?></p>
                    <p><?php echo wp_kses(
                        sprintf(
                            __( 'Visit <a href="%s" target="_blank">omahoung.com</a> for more information about our projects and services.', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ),
                            esc_url( $this->omahoung_default_url )
                        ),
                        array( 'a' => array( 'href' => array(), 'target' => array() ) )
                    ); ?></p>


                </div>
        
                <div class="tlt-section">
                    <h2><?php esc_html_e( 'Get Support', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ); ?></h2>
                    <p><?php echo wp_kses(
                        sprintf(
                            __( 'Need assistance with the plugin? Reach out through our <a href="%s#contact" target="_blank">contact form</a>.', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ),
                            esc_url( $this->omahoung_default_url )
                        ),
                        array( 'a' => array( 'href' => array(), 'target' => array() ) )
                    ); ?></p>
                </div>

                <div class="tlt-section">
                    <h2><?php esc_html_e( 'Share Your Ideas', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ); ?></h2>
                    <p><?php esc_html_e( 'We value your input! If you have any suggestions, you can connect with us using the above contact form link.', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ); ?></p>
                </div>
        
                <div class="tlt-section">
                    <h2><?php esc_html_e( 'Donations', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ); ?></h2>
                    <p><?php esc_html_e( 'Your support is invaluable, sustaining ongoing development and maintenance of this free plugin. Creating and upkeeping this plugin requires significant time and effort, and your contributions play a crucial role in its success.', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ); ?></p>
                    <p><?php esc_html_e( 'If you\'d like to make a donation and contribute to the continued enhancement of this tool, you can do so via PayPal:', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ); ?></p>
                    <form action="https://www.paypal.com/donate" method="post" target="_blank">
                        <input type="hidden" name="business" value="UTCYT6CZS4WHG" />
                        <input type="hidden" name="no_recurring" value="1" />
                        <input type="hidden" name="currency_code" value="EUR" />
                        <input type="hidden" name="item_name" value="Omahoung" />
                        <input type="image" src="<?php echo esc_url( plugins_url( 'assets/img/btn_donateCC_LG.gif', dirname( __FILE__ ) ) ); ?>" border="0" name="submit" title="<?php esc_html_e( 'Donate via PayPal', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ); ?>" alt="<?php esc_html_e( 'Donate with PayPal button', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ); ?>" />
                    </form>
                </div>
            </div>

            <!-- Privacy Policy -->
            <div id="tlt-privacy-policy" class="tlt-tab-content">
                <div class="tlt-section">
                    <h2><?php esc_html_e( 'Privacy Policy', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ); ?></h2>
                    <p><?php esc_html_e( 'Omahoung is committed to protecting your privacy and providing a positive user experience. This Privacy Policy outlines the information we collect, and how we use it. By using this plugin, you consent to the practices described in this policy.', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ); ?></p>
                </div>
                <div class="tlt-section">
                    <h2><?php esc_html_e( 'Information We Collect', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ); ?></h2>
                    <p><?php esc_html_e( 'We collect basic information, such as your website domain, plugin version (including Transposh version), WordPress version, and PHP version.', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ); ?></p>
                </div>
                <div class="tlt-section">
                    <h2><?php esc_html_e( 'How We Use Your Information', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ); ?></h2>
                    <p><?php esc_html_e( 'The collected information help us to understand the number of active installations, and to ensure that our plugin works seamlessly with different WordPress and PHP setups.', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ); ?></p>
                </div>
                <div class="tlt-section">
                    <h2><?php esc_html_e( 'Contact Us', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ); ?></h2>
                    <p><?php echo wp_kses(
                        sprintf(
                            __( 'If you have any questions or concerns about this Privacy Policy, please <a href="%s#contact" target="_blank">contact us</a>.', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ),
                            esc_url( $this->omahoung_default_url )
                        ),
                        array( 'a' => array( 'href' => array(), 'target' => array() ) )
                    ); ?></p>
                </div>
            </div>    

        </div>
        <?php
    }    
}