<?php
/**
 * Activation Deactivation Class
 *
 * @package Transposh_Language_Toolbox
 */

namespace Transposh_Language_Toolbox;

class Activation_Deactivation {
    /**
     * Activation hook callback.
     */
    public function activate() {

        $default = array();

        // Create default options
		if ( ! get_option( TRANSPOSH_TOOLBOX_OPTION_NAME ) ) {
			update_option( TRANSPOSH_TOOLBOX_OPTION_NAME, $default );
		}

        // Track plugin activation
        $this->track_activation();
    }

    /**
     * Deactivation hook callback.
     */
    public function deactivate() {
        // Track plugin deactivation
        $this->track_deactivation();
    }

    /**
     * Track plugin activation and send data to the repository.
     */
    private function track_activation() {
        // Get the WordPress version
        global $wp_version;

        // Get the domain
        $domain = home_url();

        // Get the PHP version
        $php_version = phpversion();

        // Get the plugin version
        $plugin_slug = plugin_basename( plugin_dir_path( __DIR__, 1 ) );
        $plugin_dir = plugin_dir_path( dirname( __FILE__, 1 ) );
        $plugin_file = $plugin_dir . $plugin_slug . '.php';
        $plugin_data = get_file_data( $plugin_file, array( 'Version' => 'Version' ) );
        $version = strval( $plugin_data['Version'] );

        // Prepare the data to be sent to your info.php file
        $data = array(
            'action' => 'activation',
            'plugin_name' => $plugin_slug,
            'version' => $version,
            'domain' => $domain,
            'transposh_version' => TRANSPOSH_TOOLBOX_TRANSPOSH_VERSION,
            'wp_version' => $wp_version,
            'php_version' => $php_version,
        );

        // Send the data to repository using a POST request
        $response = wp_remote_post( TRANSPOSH_TOOLBOX_PLUGIN_REPOSITORY_URL . 'plugin-logger.php', array(
            'method' => 'POST',
            'body' => $data,
        ) );

        // Can check the response if needed
        if ( is_wp_error( $response ) ) {
            // Handle the error if the request fails
        }
    }

    /**
     * Track plugin activation and send data to the repository.
     */
    private function track_deactivation() {
        // Get the domain
        $domain = home_url();

        // Get the plugin slug
        $plugin_slug = plugin_basename( plugin_dir_path( __DIR__, 1 ) );

        // Prepare the data to be sent to your info.php file
        $data = array(
            'action' => 'deactivation',
            'plugin_name' => $plugin_slug,
            'domain' => $domain,
        );

        // Send the data to repository using a POST request
        $response = wp_remote_post( TRANSPOSH_TOOLBOX_PLUGIN_REPOSITORY_URL . 'plugin-logger.php', array(
            'method' => 'POST',
            'body' => $data,
        ) );

        // Can check the response if needed
        if ( is_wp_error( $response ) ) {
            // Handle the error if the request fails
        }
    }
}