<?php
/**
 * Base Utilities Class
 *
 * @package Transposh_Language_Toolbox
 */

namespace Transposh_Language_Toolbox;

/**
 * Base class with shared variables and functions
 */
class Base_Utilities {

    // Transposh Language Toolbox options
    public $tlt_options;

    // Transposh options
    public $tp_options;

    // Transposh options main variables
    public $default_language;
    public $viewable_languages;

    /**
     * Constructor
     */  
    public function __construct() {
        $this->tlt_options = get_option( TRANSPOSH_TOOLBOX_OPTION_NAME );
        $this->tp_options = get_option( TRANSPOSH_TOOLBOX_TP_OPTION_NAME );
        $this->default_language = isset( $this->tp_options['default_language'] ) ? $this->tp_options['default_language'] : '';
        $this->viewable_languages = isset( $this->tp_options['viewable_languages'] ) ? $this->tp_options['viewable_languages'] : '';
    }

    /**
     * Get the active language based on the current URL
     *
     * @return string The active language code
     */
    public function get_active_language() {
        //$current_url = home_url( add_query_arg( $_GET, $GLOBALS['wp']->request ) );
        $current_url = add_query_arg( $_SERVER['QUERY_STRING'], '', home_url( $GLOBALS['wp']->request ) );
        $language = \transposh_utils::get_language_from_url( $current_url, home_url() );
        if ( $language ) {
            return $language;
        }
        return $this->default_language;
    }

    /**
     * Simple minification for CSS and JS by removing tabs, newlines, and extra spaces.
     *
     * @param string $content The content to be minified.
     * @param int $remove_spaces The number of spaces to remove (default is 0).
     * @return string The minified content.
     */
    function simple_minify( $content, $remove_spaces = 0 ) {
        if ( $remove_spaces > 0 ) { // For admin we keep the formating
            return preg_replace('/^\s{' . $remove_spaces . '}/m', '', $content);
        } else {
            $content = preg_replace('/\/\*(.*?)\*\//s', '', $content);
            return preg_replace( '/\s+/', ' ', str_replace( array( "\t", "\n", "\r"), '', $content ) );
        }
    }
}