<?php
/**
 * Translation Editor Class
 *
 * @package Transposh_Language_Toolbox
 */

namespace Transposh_Language_Toolbox;

use WP_List_Table;

if ( !class_exists('WP_List_Table' ) ) {
    require_once( ABSPATH . 'wp-admin/includes/class-wp-list-table.php' );
}

/**
 * Class for rendering and updating the translation table.
 */
class Translation_Editor extends WP_List_Table {

    /**
     * Constructor.
     */
    public function __construct() {
        parent::__construct( array(
            'singular' => __( 'translation', TRANSPOSH_EDITOR_PER_PAGE_DEFAULT ), // singular name of the listed records
            'plural'   => __( 'translations', TRANSPOSH_EDITOR_PER_PAGE_DEFAULT ), // plural name of the listed records
            'ajax'     => true // does this table support ajax?

        ) );
    }

    /**
     * Render the Translations tab content.
     */  
    public function render_translations_tab() {
        ?>
        <div class="tlt-wrap">
            <div class="tlt-menu">
                <a href="#" class="tlt-menu-item active" data-tab="tlt-translation-editor"><?php esc_html_e( 'Translation Editor', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ); ?></a>
                <span class="tlt-menu-divider">|</span>
                <a href="#" class="tlt-menu-item" data-tab="tlt-editor-utilities"><?php esc_html_e( 'Utilities', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ); ?></a>
                <span class="tlt-menu-divider">|</span>                
                <a href="#" class="tlt-menu-item" data-tab="tlt-editor-help"><?php esc_html_e( 'Help', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ); ?></a>
            </div>

            <!-- Translator Editor section -->
            <div id="tlt-translation-editor" class="tlt-tab-content active">
                <form id="tlt-translations-editor-form" method="get">
                <input type="hidden" name="page" value="transposh-language-toolbox">
                <?php 
                $this->prepare_items();
                $this->search_box( esc_html__( 'Search', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ), 's' );
                $this->display();
                ?>
                </form>
            </div>

            <!-- Utilities section -->
            <div id="tlt-editor-utilities" class="tlt-tab-content">
                <div class="actions">
                    <?php
                    echo( '<p>' . esc_html__( 'For detailed information and instructions, please refer to the Help section.', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ) . '</p>' );
                    // Identical Translations search button
                    echo '<a href="' . esc_url( add_query_arg( 'filter-identical', '1' ) ) . '" class="button button-secondary">' . esc_html__( 'Identical Translations Finder', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ) . '</a>';
                    echo '<a href="#" id="tlt-export-csv-button" class="button button-secondary">' . esc_html__('Export as CSV', TRANSPOSH_TOOLBOX_TEXT_DOMAIN) . '</a>';
                    ?>
                </div>
            </div>
            
            <!-- Editor help -->
            <div id="tlt-editor-help" class="tlt-tab-content">
                <div class="tlt-section">
                    <h2><?php esc_html_e( 'Translation Editor', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ); ?></h2>
                    <p><?php echo wp_kses( __( '<strong>Edit Content:</strong> Simply click on cells in the "Original" and "Translated" columns to edit content. Changes are automatically saved when you click outside the cell or press Enter.', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ), array ( 'strong' => array() ) ); ?></p>
                    <p><?php echo wp_kses( __( '<strong>Filter and Search:</strong> Use filters to narrow down your view by language or source (Human or Engine). Use the search field to find specific strings.', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ), array ( 'strong' => array() ) ); ?></p>
                    <p><?php echo wp_kses( __( '<strong>Delete Translations:</strong> Select translations with the checkboxes and use bulk delete for removal.', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ), array ( 'strong' => array() ) ); ?></p>
                    <h2><?php esc_html_e( 'Utilities', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ); ?></h2>
                    <p><?php echo wp_kses( __( '<strong>Identical Translations Finder:</strong> This displays rows where translation matches the original content (you may want to remove these rows). To use this function with filters, first set your filter options in the Translation Editor and click the "Filter" button, then, in Utilities, click the "Identical Translations Finder" button.', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ), array ( 'strong' => array() ) ); ?></p>
                    <p><?php echo wp_kses( __( '<strong>Export as CSV:</strong> Before exporting, you can apply filters and sorting; these settings will be used for the generated CSV file. The caret (^) character serves as column delimiter. When importing into applications like Excel, remember to specify this delimiter. To ensure proper CSV formatting, the export process replaces any newline character (\n) within strings with a space.', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ), array ( 'strong' => array() ) ); ?></p>
                </div>
            </div>
        </div>
        <?php
    }    

    /**
     * Prepare the items for the table.
     */
    public function prepare_items() {
        global $wpdb;
        $table_name = $wpdb->prefix . 'translations';

        $column_hidden = ( is_array( get_user_meta( get_current_user_id(), TRANSPOSH_EDITOR_COLUMNS_FILTER_OPTION_NAME, true)) ) ? get_user_meta( get_current_user_id(), TRANSPOSH_EDITOR_COLUMNS_FILTER_OPTION_NAME, true) : array();

        $columns               = $this->get_columns();
		$sortable              = $this->get_sortable_columns();
		$hidden                = $column_hidden;
		$primary               = 'original';
		$this->_column_headers = array( $columns, $hidden, $sortable, $primary );

        // Process bulk actions
        $this->process_actions();

        // Retrieve the data from the database
        $translations = $this->get_translations( $table_name );
        $this->items = $translations;

        // Set up the table columns
        $this->set_sortable_columns( $sortable );

        // Set up the data for the table
        $current_page = $this->get_pagenum();
        $total_items = count( $translations );
        $per_page = $this->get_items_per_page( TRANSPOSH_EDITOR_PER_PAGE_OPTION_NAME, TRANSPOSH_EDITOR_PER_PAGE_DEFAULT );
        $this->set_pagination_args( array(
            'total_items' => $total_items,
            'per_page'    => $per_page,
        ) );

        // Slice the translations array based on the current page and items per page
        $offset = ( $current_page - 1 ) * $per_page;
        $this->items = array_slice( $translations, $offset, $per_page );
    }

    /**
	 * Generates the table navigation above or below the table.
	 *
	 * @param string $which Position of the navigation, either top or bottom.
	 */
	protected function display_tablenav( $which ) {
		?>
        <div class="tablenav <?php echo esc_attr( $which ); ?>">

            <?php if ( $this->has_items() ) { ?>
                <div class="alignleft actions bulkactions">
                    <?php $this->bulk_actions( $which ); ?>
                </div>
            <?php }
            $this->extra_tablenav( $which );
            $this->pagination( $which );
            ?>

            <br class="clear" />
        </div>
		<?php
	}

	/**
	 * Overriden method to add dropdown filters column type.
	 *
	 * @param string $which Position of the navigation, either top or bottom.
	 */
    protected function extra_tablenav( $which ) {
        if ( 'top' === $which ) {
            
            // Languages filter

            global $wpdb;
            $table_name = $wpdb->prefix . 'translations';
            // Retrieve all the languages from the table
            $languages = $wpdb->get_col( "SELECT DISTINCT lang FROM $table_name" );
            // Create the dropdown options
            $options = array( '' => esc_html__( 'All languages', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ) );
            foreach ( $languages as $language ) {
                $options[ $language ] = $language;
            }

            echo '<div class="alignleft actions">';

            // Retrieve the selected language
            $selected_language = isset( $_REQUEST['filter-language'] ) ? $_REQUEST['filter-language'] : '';
            // Set up the dropdown arguments
            $filter_dropdown_arg = array(
                'options'   => $options,
                'container' => array(
                    'class' => 'alignleft',
                ),
                'label'     => array(
                    'class'      => 'screen-reader-text',
                    'inner_text' => esc_html__( 'Filter by language', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ),
                ),
                'select'    => array(
                    'name'     => 'filter-language',
                    'id'       => 'filter-language-id',
                    'selected' => $selected_language,
                ),
            );
            $this->html_dropdown( $filter_dropdown_arg );


            // Sources filter

            // Retrieve the selected items
            $selected_source = ( isset( $_REQUEST['filter-source'] ) ? $_REQUEST['filter-source'] : '' );
            // Set the select default label 
            $all_sources = esc_html__( 'All sources', TRANSPOSH_TOOLBOX_TEXT_DOMAIN );

            $filter_dropdown_arg = array (
				'options'   => array( 
                    ''          =>  $all_sources, 
                    '0'         => 'Human', 
                    '1'         => 'Engine' 
                ),
				'container' => array(
					'class' => 'alignleft',
				),
				'label'     => array(
					'class'      => 'screen-reader-text',
					'inner_text' => esc_html__( 'Filter by source', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ),
				),
				'select'    => array(
					'name'     => 'filter-source',
					'id'       => 'filter-source-id',
					'selected' => $selected_source,
				),
            );

            $this->html_dropdown( $filter_dropdown_arg );

            submit_button( esc_html__( 'Filter', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ), 'secondary tlt-button-doaction', 'filter-action', false );
            
            echo '</div>';
        }
    }

    /**
	 * Navigation dropdown HTML generator.
	 *
	 * @param array $args Argument array to generate dropdown.
	 */
	private function html_dropdown( $args ) {
		?>
		<div class="<?php echo( esc_attr( $args['container']['class'] ) ); ?>">
			<label
				for="<?php echo( esc_attr( $args['select']['id'] ) ); ?>"
				class="<?php echo( esc_attr( $args['label']['class'] ) ); ?>">
			</label>
			<select
				name="<?php echo( esc_attr( $args['select']['name'] ) ); ?>"
				id="<?php echo( esc_attr( $args['select']['id'] ) ); ?>">
				<?php
				foreach ( $args['options'] as $id => $title ) {
					?>
					<option
					<?php if ( $args['select']['selected'] === strval( $id ) ) {?>
						selected="selected"
					<?php } ?>
					value="<?php echo( esc_attr( $id ) ); ?>">
					<?php echo esc_html( $title ); ?>
					</option>
					<?php
				}
				?>
			</select>
		</div>
		<?php
	}

    /**
     * Retrieve translations from the database with sorting.
     *
     * @param string $table_name Table name.
     * @param string $orderby    Sort column.
     * @param string $order      Sort order.
     * @return array  Translations data array.
     */
    protected function get_translations( $table_name ) {
        global $wpdb;

        // Get the search term
        $search_term = isset( $_REQUEST['s'] ) ? sanitize_text_field( $_REQUEST['s'] ) : '';

        // Get the selected language and source filter values
        $selected_language = isset( $_REQUEST['filter-language'] ) ? $_REQUEST['filter-language'] : '';
        $selected_source = isset( $_REQUEST['filter-source'] ) ? $_REQUEST['filter-source'] : '';

        // Check if the identical translations filter is requested
        $identical_filter = isset( $_GET['filter-identical'] ) && $_GET['filter-identical'] === '1';
    
        // Get the sort values
        $orderby = isset( $_REQUEST['orderby'] ) ? sanitize_key( $_REQUEST['orderby'] ) : 'timestamp';
        $order = isset( $_REQUEST['order'] ) && in_array( $_REQUEST['order'], array( 'asc', 'desc' ) ) ? $_REQUEST['order'] : 'desc';
    
        // Add the orderby and order values to the form
        echo '<input type="hidden" name="orderby" value="' . esc_attr( $orderby ) . '" />';
        echo '<input type="hidden" name="order" value="' . esc_attr( $order ) . '" />';


        // Set up the WHERE clause
        $where = '';
      
        // Check if filter_language is set and not empty
        if ( !empty( $selected_language ) ) {
            $where = $wpdb->prepare(
                ' WHERE lang = %s',
                $selected_language
            );
        }

        // Check if filter_source is set and not empty
        if ( $selected_source !== '' ) {
            $where .= $wpdb->prepare(
                ( empty( $where ) ? ' WHERE' : ' AND' ) . ' source = %s',
                $selected_source
            );
        }

        // Check if identical filter is enabled
        if ( $identical_filter ) {
            $where .= ( empty( $where ) ? ' WHERE' : ' AND' ) . ' original = translated';
        }

        // Check if search term is set and not empty
        if ( ! empty( $search_term ) ) {
            $where .= $wpdb->prepare(
                ( empty( $where ) ? ' WHERE' : ' AND' ) . ' (original LIKE %s OR translated LIKE %s)',
                '%' . $wpdb->esc_like( $search_term ) . '%',
                '%' . $wpdb->esc_like( $search_term ) . '%'
            );
        }

        $query = "SELECT * FROM $table_name $where ORDER BY $orderby $order";
        //error_log('Generated Query: ' . $query);

        return $wpdb->get_results( $query );
    }
 
    /**
     * Render the column content.
     */
    public function column_default( $item, $column_name ) {
        switch ( $column_name ) {
            case 'language':
                return $item->lang;
            case 'translated_by':
            case 'source':
            case 'timestamp':
                return $item->$column_name;
            case 'original':
            case 'translated':
                //$content = $item->$column_name;
                $content = htmlspecialchars( $item->$column_name );
                $before_value = esc_attr( $item->$column_name );
                $original_value = esc_attr( $item->original );
                $language = esc_attr( $item->lang );
                return sprintf(
                    '<span class="tlt-editable-cell" contenteditable="true" column-updated="%s" before-value="%s" data-original="%s" data-language="%s">%s</span>',
                    $column_name,
                    $before_value,
                    $original_value,
                    $language,
                    $content
                );
            default:
                return '';
        }
    }

    /**
     * Set the number of items to display per page.
     *
     * @param int $per_page Number of items per page.
     */
    public function set_items_per_page( $per_page ) {
        $this->per_page = $per_page;
    }

    /**
     * Get the number of items to display per page.
     *
     * @param string $option         Option name.
     * @param int    $default_value  Default items per page.
     * @return int  Items per page.
     */
    protected function get_items_per_page( $option, $default_value = TRANSPOSH_EDITOR_PER_PAGE_DEFAULT ) {
        $per_page = get_user_meta( get_current_user_id(), $option, true ); // Retrieve the per page value from user meta
    
        if ( empty( $per_page ) || $per_page < 1 ) {
            $per_page = $default_value;
        }

        return $per_page;
    }

    /**
     * Get the current page number.
     *
     * @return int  Current page number.
     */
    public function get_pagenum() {
        return isset( $_GET['paged'] ) ? absint( $_GET['paged'] ) : 1;
    }

    /**
     * Define list columns (table headers).
     * 
     * @return array Table columns
     */
    public function get_columns() {
        $columns = array(
            'cb' => '<input type="checkbox"/>',
            'original' => '<span class="tlt-column-editable-title">' . esc_html__( 'Original', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ) . '</span>',
            'language' => esc_html__( 'Language', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ),
            'translated' => '<span class="tlt-column-editable-title">' . esc_html__( 'Translated', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ) . '</span>',
            'translated_by' => esc_html__( 'Translated By', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ),
            'source' => esc_html__( 'Source', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ),
            'timestamp' => esc_html__( 'Timestamp', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ),
        );

        return $columns;
    }

    /**
	 * Column cb.
	 *
	 * @param  array array|object $item Item data.
	 * @return string
	 */
	public function column_cb( $item ) { 
        $unique_id = $this->item_key( $item );
		return sprintf(
			'<input type="checkbox" name="%1$s_id[]" value="%2$s" />',
			esc_attr( $this->_args['singular'] ),
			esc_attr( $unique_id )
		);
	}

    /**
     * Generates a unique key for an item based on its original text and language.
     *
     * @param array $item The item to generate the key for.
     *                    It should have 'original' and 'lang' keys.
     * @return string The generated key for the item.
     */
    public static function item_key( $item ) {
        // Check if $item is an object and convert it to an array
        if ( is_object( $item ) ) {
            $item = ( array ) $item;
        }
        // Concatenate the original text and language with a comma separator
        $key = $item['lang'] . ',' . $item['original'];
        // Encode the key using base64 encoding for uniqueness and readability
        return base64_encode( $key );
    }

     /**
     * Define sortable columns.
     * 
     * @return array Sortable columns
     */
    public function get_sortable_columns() {
        $sortable_columns = array(
            'original' => array( 'original', false ),
            //'language' => array( 'lang', false ),
            'translated' => array( 'translated', false ),
            'translated_by' => array( 'translated_by', false ),
            //'source' => array( 'source', false ),
            'timestamp' => array( 'timestamp', false ),
        );

        return $sortable_columns;
    }

    /**
	 * Get bulk actions.
	 *
	 * @return array
	 */
	public function get_bulk_actions() {
		return array(
			'bulk-delete' => esc_html__( 'Delete', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ),
		);
	}

	/**
	 * Process bulk actions.
	 */
    public function process_actions() {
        //echo "Process action: "; print_r($_REQUEST);  // For debug only
        if ( isset( $_REQUEST['action'] ) ) {
            if ( $_REQUEST['action'] === 'bulk-delete' && isset( $_REQUEST['translation_id'] ) ) {
                // Bulk delete action
                $selected_items = $_REQUEST['translation_id'];
                foreach ( $selected_items as $item ) { 
                    // Extract the item details
                    list( $lang, $original ) = explode( ',', base64_decode( $item ), 2 );
                    // Perform the deletion operation
                    $result = $this->delete_item( $original, $lang );
                    if ( $result ) {
                        echo esc_html__( 'Deleted item:', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ) . '<br>';
                        echo 'Original: ' . $original . ', Lang: ' . $lang . '<br>';
                    } else {
                        echo esc_html__( 'Failed to delete item:', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ) . '<br>';
                        echo 'Original: ' . $original . ', Lang: ' . $lang . '<br>';
                    }
                }
            }
        } 
    }

    /**
     * Delete an item.
     *
     * @param string $original The original value.
     * @param string $lang The language.
     * @return bool True if the item was deleted successfully, false otherwise.
     */
    private function delete_item( $original, $lang ) {
        //echo "original: "; print_r( $original ); echo "lang: "; print_r( $lang ); // For debug only
        global $wpdb;
        $table_name = $wpdb->prefix . 'translations';

        // Construct the WHERE clause for deletion
        $where = array(
            'original' => $original,
            'lang' => $lang,
        );

        // Perform the deletion operation
        $result = $wpdb->delete( $table_name, $where );

        return $result !== false;
    }

    /**
     * AJAX action callback for updating the translation.
     */
    public static function update_translation_callback() {
        if ( isset( $_POST['data'] ) ) {
            $data = json_decode( stripslashes( $_POST['data'] ), true );

            // Debugging statements
            // error_log( 'Received JSON data: ' . $_POST['data'] );
            // error_log( 'Decoded Data: ' . print_r( $data, true ) ); 

            $currentValue = wp_kses_post( $data['currentValue'] ); //Sanitize
            //$currentValue = str_replace(' & ', ' &amp; ', $currentValue);
            $originalValue = wp_kses_post( $data['originalValue'] );
            //$originalValue = str_replace(' & ', ' &amp; ', $originalValue);
            $currentValue = esc_sql( wp_unslash( $data['currentValue'] ) );
            $originalValue = esc_sql( wp_unslash( $data['originalValue'] ) );
            
            $language = esc_sql( $data['language'] );
            $columnUpdated = esc_sql( $data['columnUpdated'] );
            $nonce = esc_sql( $data['tltNonce'] );
    
            // error_log( 'Current Value: ' . print_r( $data['currentValue'], true ) );
            // error_log( 'Original Value: ' . print_r( $data['originalValue'], true ) );

            // Verify the nonce
            if ( ! wp_verify_nonce( $nonce, 'tlt_translation_editor_action' ) ) {
                $response = array(
                    'success' => false,
                    'message' => esc_html__( 'Invalid nonce. Failed to update translation!', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ),
                    'error' => esc_html__( 'Nonce verification failed.', TRANSPOSH_TOOLBOX_TEXT_DOMAIN )
                );
                wp_send_json( $response );
                wp_die();
            }

            $key = array (
                'lang' => $language,
                'original' => $originalValue,
            );

            // Update the record in the database
            global $wpdb;
            $table_name = $wpdb->prefix . 'translations';

            // Get the current user ID
            $user_id = get_current_user_id();

            // Construct the update query
            $update_data = array(
                'translated_by' => $user_id,
                'source' => 0,
            );
            $where = array(
                'original' => $originalValue,
                'lang' => $language,
            );

            if ( $columnUpdated === 'original' ) {
                $update_data['original'] = $currentValue;
                $key['original'] = $currentValue;
            } elseif ( $columnUpdated === 'translated' ) {
                $update_data['translated'] = $currentValue;
            }
            // Debugging statements for query elements
            // error_log( 'Key: ' . print_r( $key, true ) );
            // error_log( 'Update Data: ' . print_r( $update_data, true ) );
            // error_log( 'Where Clause: ' . print_r( $where, true ) );
            $updated_rows = $wpdb->update( $table_name, $update_data, $where );

            if ( $updated_rows !== false && $updated_rows > 0 ) {
                // Update successful
                $newKey = self::item_key( $key );
                $response = array(
                    'success' => true,
                    'message' => esc_html__( 'Translation updated successfully!', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ),
                    'key' => $newKey,
                    'user_id' => $user_id,
                );
            } elseif ( $updated_rows === 0 ) {
                // No rows were updated
                $response = array(
                    'success' => false,
                    'message' => esc_html__( 'No rows were updated. Data not found in the database.', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ),
                );
                http_response_code( 400 ); // Set a custom status code indicating a client-side error
            } else {
                // Update failed
                $response = array(
                    'success' => false,
                    'message' => esc_html__( 'Failed to update translation!', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ),
                    'error' => $wpdb->last_error  // Include the database error message in the response
                );
                http_response_code( 400 ); // Set a custom status code indicating a client-side error
            }
            wp_send_json( $response );
        }
        wp_die();
    }

    /**
     * Ajax callback for CSV export.
     */
    public static function export_csv_callback() {
        // Check for nonce
        if ( ! isset( $_GET['nonce'] ) || ! wp_verify_nonce( $_GET['nonce'], 'tlt_editor_export_csv_nonce' ) ) {
            $response = array(
                'success' => false,
                'message' => esc_html__( 'Invalid nonce. Failed to download CSV!', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ),
                'error' => esc_html__( 'Nonce verification failed.', TRANSPOSH_TOOLBOX_TEXT_DOMAIN )
            );
            http_response_code( 400 ); // Set a custom status code indicating a client-side error
            wp_send_json( $response );
            wp_die();
        }

        // Perform the CSV export logic here
        if ( isset( $_GET['export_csv'] ) && $_GET['export_csv'] === '1' ) { 
            // Retrieve the data using the get_translations() function
            $translations = self::ajax_get_translations();

            // Set CSV headers with UTF-8 encoding
            header( 'Content-Type: text/csv; charset=utf-8' );
            header( 'Content-Disposition: attachment; filename="translations.csv"' );

            // Output CSV data
            $output = fopen( 'php://output', 'w' );

            // Specify tab as the delimiter character
            $delimiter = "^";

            // Write CSV headers manually
            $headerRow = array( 'Original', 'Language', 'Translated', 'Translated By', 'Source', 'Timestamp' );
            fwrite( $output, implode( $delimiter, $headerRow ) . "\n");

            // Loop through translations and write each row to CSV
            foreach ( $translations as $translation ) {
                $rowData = array(
                    str_replace( "\n", ' ', $translation->original ),
                    $translation->lang,
                    str_replace( "\n", ' ', $translation->translated ),
                    $translation->translated_by,
                    $translation->source,
                    $translation->timestamp
                );
                // Manually join the row data with the custom delimiter
                $row = implode( $delimiter, $rowData );
                // Write the formatted row to the output file
                fwrite( $output, $row . "\n" );
            }

            fclose( $output );
            exit; // Prevent further output

        } else {
            $response = array(
                'success' => false,
                'message' => esc_html__( 'CSV export failed!', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ),
                'error' => esc_html__( 'Request failed.', TRANSPOSH_TOOLBOX_TEXT_DOMAIN )
            );
            http_response_code( 400 ); // Set a custom status code indicating a client-side error
            wp_send_json( $response );
        }
        wp_die();
    }

    /**
     * Retrieve translations from the database with sorting.
     *
     * @return array  Translations data array.
     */
    private static function ajax_get_translations() {
        global $wpdb;
        $table_name = $wpdb->prefix . 'translations';

        $referer_url = $_SERVER['HTTP_REFERER'];
        $parsed_url = parse_url( $referer_url );
        parse_str( $parsed_url['query'], $query_args );
        
        // Now we can access the url parameters using $query_args array
        $search_term = isset( $query_args['s']) ? sanitize_text_field( $query_args['s'] ) : '';
        $selected_language = isset( $query_args['filter-language'] ) ? $query_args['filter-language'] : '';
        $selected_source = isset( $query_args['filter-source'] ) ? $query_args['filter-source'] : '';
        $identical_filter = isset( $query_args['filter-identical'] ) && $query_args['filter-identical'] === '1';
        $orderby = isset( $query_args['orderby']) ? sanitize_key( $query_args['orderby'] ) : 'timestamp';
        $order = isset( $query_args['order'] ) && in_array( $query_args['order'], array( 'asc', 'desc' ) ) ? $query_args['order'] : 'desc';

        // Set up the WHERE clause
        $where = '';
      
        // Check if filter_language is set and not empty
        if ( !empty( $selected_language ) ) {
            $where = $wpdb->prepare(
                ' WHERE lang = %s',
                $selected_language
            );
        }

        // Check if filter_source is set and not empty
        if ( $selected_source !== '' ) {
            $where .= $wpdb->prepare(
                ( empty( $where ) ? ' WHERE' : ' AND' ) . ' source = %s',
                $selected_source
            );
        }

        // Check if identical filter is enabled
        if ( $identical_filter ) {
            $where .= ( empty( $where ) ? ' WHERE' : ' AND' ) . ' original = translated';
        }

        // Check if search term is set and not empty
        if ( ! empty( $search_term ) ) {
            $where .= $wpdb->prepare(
                ( empty( $where ) ? ' WHERE' : ' AND' ) . ' (original LIKE %s OR translated LIKE %s)',
                '%' . $wpdb->esc_like( $search_term ) . '%',
                '%' . $wpdb->esc_like( $search_term ) . '%'
            );
        }

        $query = "SELECT * FROM $table_name $where ORDER BY $orderby $order";
        //error_log('Generated Query: ' . $query);

        return $wpdb->get_results( $query );
    }
}