<?php
/**
 * Plugin Name: Transposh Language Toolbox
 * Plugin URI:  https://www.omahoung.com
 * Description: A plugin for Transposh with translation editing, language switcher shortcodes, custom language support, and others...
 * Version:     0.5.1
 * Author:      Omahoung
 * Author URI:  https://www.omahoung.com
 * Text Domain: transposh-language-toolbox
 * Domain Path: /languages
 */

// If this file is called directly, abort!!!
defined( 'ABSPATH' ) or die( 'Hey, you can\'t access this file, you silly human!' );

// Require once autoload the classes
if ( file_exists( dirname( __FILE__ ) . '/autoloader/autoload.php' ) ) {
	require_once dirname( __FILE__ ) . '/autoloader/autoload.php';
}
// Include our constants file
require_once plugin_dir_path( __FILE__ ) . 'constants.php';
// Include the Custom_Languages class
require_once dirname( __FILE__ ) . '/includes/Custom_Languages.php';

// Instantiate the activation and deactivation class.
$transposh_language_toolbox_activation_deactivation = new Transposh_Language_Toolbox\Activation_Deactivation();
register_activation_hook( __FILE__, array( $transposh_language_toolbox_activation_deactivation, 'activate' ) );
register_deactivation_hook( __FILE__, array( $transposh_language_toolbox_activation_deactivation, 'deactivate' ) );

/**
 * Load the plugin textdomain
 */
function transposh_language_toolbox_txtdomain() {
	load_plugin_textdomain( TRANSPOSH_TOOLBOX_TEXT_DOMAIN, false, dirname( plugin_basename( __FILE__ ) ) . '/languages' );
}
add_action( 'init', 'transposh_language_toolbox_txtdomain' );

// Add custom languages to transposh constants
function transposh_language_toolbox_modify_languages() {
    \Transposh_Language_Toolbox\Custom_Languages::modify_transposh_languages();
}
add_action('plugin_loaded', 'transposh_language_toolbox_modify_languages', 0);

/**
 * Initialize the Transposh Translation Editor plugin
 */
function transposh_language_toolbox_init() {
	// Check if the 'transposh_plugin' class exists
	if ( class_exists( 'transposh_plugin' ) ) {
		// Instantiate the main plugin class.
		$transposh_language_toolbox = new Transposh_Language_Toolbox\Language_Toolbox_Main();
		$transposh_language_toolbox->init();
	} else {
		// Display a notice if the 'transposh_plugin' class is not found
		add_action( 'admin_notices', 'transposh_language_toolbox_missing_notice' );
	}
}
add_action( 'plugins_loaded', 'transposh_language_toolbox_init' ); // Hook into the plugins_loaded action

/**
 * Display a notice if the 'transposh_plugin' class is not found
 */
function transposh_language_toolbox_missing_notice() {
	$class = 'notice notice-error';
	$message = esc_html__( 'The Transposh Language Toolbox requires the Transposh plugin to be installed and activated. Please install and activate the Transposh plugin.', TRANSPOSH_TOOLBOX_TEXT_DOMAIN );
	printf( '<div class="%1$s"><p>%2$s</p></div>', $class, $message );
}