<?php
/**
* Trigger this file on Plugin uninstall
*
* @package Transposh_Language_Toolbox
*/

if ( ! defined( 'WP_UNINSTALL_PLUGIN' ) ) {
	die;
}

global $wpdb;

// Include the constants file to get access to the defined constants
require_once( plugin_dir_path( __FILE__ ) . 'constants.php' );

// Delete user meta for all users
$wpdb->query( "
    DELETE FROM {$wpdb->usermeta}
    WHERE meta_key IN (
        '" . TRANSPOSH_EDITOR_PER_PAGE_OPTION_NAME . "',
        '" . TRANSPOSH_EDITOR_COLUMNS_FILTER_OPTION_NAME . "'
    )
" );

// Delete custom languages in Transposh options
$options = get_option( TRANSPOSH_TOOLBOX_OPTION_NAME );
$custom_languages = isset( $options['custom_languages'] ) ? $options['custom_languages'] : array();
if ( !empty( $custom_languages ) ) {
    $transposh_options = get_option( 'transposh_options' );
    if ( false !== $transposh_options ) {
        foreach ( $custom_languages as $key => $value ) {
            if ( $key === $transposh_options['default_language'] ) {
                $transposh_options['default_language'] = '';
            }
            if ( strpos( $transposh_options['viewable_languages'], $key ) !== false ) {
                $transposh_options['viewable_languages'] = str_replace( $key . ',', '', $transposh_options['viewable_languages']);
                $transposh_options['viewable_languages'] = str_replace( ',' . $key, '', $transposh_options['viewable_languages']);
            }
            if ( strpos( $transposh_options['sorted_languages'], $key ) !== false ) {
                $transposh_options['sorted_languages'] = str_replace( $key . ',', '', $transposh_options['sorted_languages'] );
                $transposh_options['sorted_languages'] = str_replace( ',' . $key, '', $transposh_options['sorted_languages'] );
            }
        }
        // Save the modified transposh_options back to the database
        update_option( 'transposh_options', $transposh_options );
    }
}

// Delete plugin options
delete_option( TRANSPOSH_TOOLBOX_OPTION_NAME );

// Remove translator role
$custom_role = get_role( 'tlt_translator' );
if ( $custom_role ) {
    // Role exists, remove it
    remove_role( 'tlt_translator' );
    // Remove capability from all users with the "tlt_translator" role
    $users = get_users( array( 'role' => 'tlt_translator' ) );
    foreach ( $users as $user ) {
        $user->remove_cap( 'tlt_translator' );
    }
}

// Track plugin deletion
track_deletion();

/**
 * Track plugin deletion and send data to the repository.
 */
function track_deletion() {
    // Get the domain
    $domain = home_url();

    // Get the plugin slug
    $plugin_slug = plugin_basename( plugin_dir_path( __FILE__ ) );

    // Prepare the data to be sent to your info.php file
    $data = array(
        'action' => 'deletion',
        'plugin_name' => $plugin_slug,
        'domain' => $domain,
    );

    // Send the data to repository using a POST request
    $response = wp_remote_post( TRANSPOSH_TOOLBOX_PLUGIN_REPOSITORY_URL . 'plugin-logger.php', array(
        'method' => 'POST',
        'body' => $data,
    ) );

    // Check the response if needed
    if ( is_wp_error( $response ) ) {
        // Handle the error if the request fails
    }
}